;;; -*- Mode: Lisp; Syntax: Common-Lisp; -*-
;;; YstokHelp - HTML-based help - LispWorks system definition
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Copyright (c) 2009-2014 Dr. Dmitry Ivanov.  All rights reserved.
;;; Website: http://lisp.ystok.ru/yhelp/
;;;
;;; The :deliver-yhelp feature controls:
;;;   on  - We are delivering the YstokHelp compiler application.
;;;   off - We are just loading run-time API into another kind of application.
;;; The :phtml feature is enough for using the help book compiler as a library.

(in-package :cl-user)

;(pushnew :ys-product *features*)
;(pushnew :deliver-yhelp *features*)
#+deliver-yhelp (pushnew :deliver *features*)
;(pushnew :debug *features*)
;(pushnew :Russian *features*)
;(removef *features* :debug)
;(yl:debug-on :help) (yl:debug-off :help)
(toggle-source-debugging #+debug t #-debug nil)

#-Russian (pushnew :English *features*)
#+deliver-yhelp (pushnew :phtml *features*)
#+phtml (pushnew :html *features*)			; preprocessor only

#+deliver (load #P"PROJECTS:ylib;ffc;defsys")
#+deliver (lw:compile-system 'YSTOK-FFC :load t)
#+deliver (load #P"PROJECTS:ylib;defsys")
#+deliver (compile-system 'YSTOK-LIBRARY :load t)
#+(and deliver deliver-yhelp) (load #P"PROJECTS:ylib;local-time;defsys")
#+(and deliver deliver-yhelp) (compile-system 'LOCAL-TIME :load t)

#+(and deliver (or (not phtml) deliver-yhelp)) (load #P"PROJECTS:ywidgets;defsys")
#+(and deliver (or (not phtml) deliver-yhelp)) (compile-system 'YWIDGETS :load t)
#+(and deliver deliver-yhelp) (load #P"PROJECTS:ygrid;defsys")
#+(and deliver deliver-yhelp) (compile-system 'YGRID :load t)

;#+(and deliver (not deliver-yhelp)) (load #P"PROJECTS:ynet;acl-compat;defsys")
;#+(and deliver (not deliver-yhelp)) (compile-system 'ACL-COMPAT :load t)

#+(and deliver (not deliver-yhelp)) (load #P"PROJECTS:ynet;uri;defsys")
#+(and deliver (not deliver-yhelp)) #+deliver (compile-system 'YSTOK-URI :load t)

#+(and deliver (not deliver-yhelp) html) (load #P"PROJECTS:ynet;html;defsys")
#+(and deliver (not deliver-yhelp) html) (compile-system 'YHTML :load t)

;; Bulk load the above three. Q: do we need net-grids.lisp?
#+(and deliver deliver-yhelp) (load #P"PROJECTS:ynet;defsys")
#+(and deliver deliver-yhelp) (compile-system 'YNET :load t)

;(compile-file #P"PROJECTS:ylib;asdlite;asdlite" :load t)
#+(and (not asdlite) phtml) (load #-debug #P"PROJECTS:ylib;asdlite;asdlite" 
                                  #+debug #P"PROJECTS:ylib;asdlite;asdlite-debug")

(defsystem YHELP (:object-pathname (lw:current-pathname
		#+(and debug (not Russian)) 	 	"bin/debug/"
		#+(and debug Russian)	 	 	"bin/debug/ru/"
		#+(and (not debug) (not Russian))	"bin/"
		#+(and (not debug) Russian)		"bin/ru/"))
 :members
  ((YSTOK-URI			:type :system)
   #+html (YHTML		:type :system)
   #+ywidgets (YWIDGETS		:type :system)
   #+deliver-yhelp (YGRID	:type :system)
   ;#-deliver-yhelp (ACL-COMPAT	:type :system)
   #+deliver-yhelp (YNET	:type :system)
   "help-package"
   ("lang/ru"		:features :Russian)		; put your language here
   "help-basics"
   ("generics"		:features :phtml)
   ("lang/ru-compiler"	:features (and :phtml :Russian)) ; strings and methods
   ("components"	:features :phtml)
   ("generating"	:features :phtml)
   ("anchors"		:features :phtml)
   ("parsing"		:features :phtml)
   ("text"		:features :phtml)
   ("context"		:features :phtml)
   ("perform"		:features :phtml)
   ("index-compiler"	:features :phtml)
   ("toc-compiler"	:features :phtml)
   ("viewer"		:features :ywidgets)		; viewer/browser control
   ("run-time"		:features :ywidgets)		; top-level utils
   ("help-widgets"	:features :ywidgets)		; ywidgets add-ons
   ("help-grid"		:features :ygrid)		; ygrid add-ons
                                                        ; The following are not exhibited
   ("help-tab"		:features (and :ys-product :ywidgets))
   ("lang/ru-product"	:features (and :deliver-yhelp :Russian))
   ("settings"		:features :deliver-yhelp)
   ("ifaces"		:features :deliver-yhelp)
   ("main"		:features :deliver-yhelp)
  )
  :rules
  ((:in-order-to :compile :all
    (:requires (:load :previous)))
   (:in-order-to :load :all
    (:requires (:load :previous)))

   (:in-order-to :load "help-tab"			; actually does not work in LW 4.4
    (:caused-by (:load "help-basics" "run-time")))

   (:in-order-to :compile ("settings" "ifaces")
    (:caused-by (:compile "ru-product")))

   (:in-order-to :load "settings"			; actually does not work
    (:caused-by (:load "ru-product")))

   (:in-order-to :load "ifaces"				; actually does not work
    (:caused-by (:load "components" "context" "ru-product")))

   (:in-order-to :load "main"				; actually does not work
    (:caused-by (:load "help-package" "components")))
) )

#+ys-product
(ys:defmodule :yhelp (1 0 :abbrev "hlp") :depends-on (:uri #+html :html :ywidgets))

;;; Stand-alone Help Preprocessor application identification

#+(and ys-product deliver-yhelp)
(setf yl:*thys-product* :help
      yl:*thys-product-name* #-Russian "YstokHelp: Help book compiler"
                             #+Russian "YstokHelp:   "
      ys:*thys-version-string* "1.7.018"
      ys:*thys-version-major*
       (if #-debug (search "beta" ys:*thys-version-string* :test 'char-equal)
           #+debug t
           (- (ys:thys-version-major (ys:thys-version-string2fixnum)))
           (ys:thys-version-major (ys:thys-version-string2fixnum)))
      (sys:product-registry-path yl:*thys-product*)
       `("Software" "Ystok"
         ,(string-capitalize yl:*thys-product*)
         ,(princ-to-string ys:*thys-version-major*))
      #+deliver-yhelp #+deliver-yhelp 
      yl::%thys-product-path% (merge-pathnames "deliver/"	; for developing
                                               (lw:pathname-location *load-pathname*))
      #+deliver-yhelp #+deliver-yhelp
      ys:*thys-product-home-http* #-Russian "http://en.ystok.ru/products/yhelp/"
                                  #+Russian "http://support.ystok.ru/doc/")

#||
(compile-system 'YHELP :load t)
(load-system 'YHELP)
(help::main)

(pushnew :test *features*)
(removef *features* :test)
(capi:find-interface 'ywi:log-interface)
||#
