;;; YstokHelp - HTML-based help - Help book definition
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Copyright (c) 2009-2012 Ystok-Systema.  All rights reserved.
;;; Website: http://lisp.ystok.ru/yhelp/
;;;
;;; Definition of the help book about YstokHelp itself.
;;;
;;; NOTES 
;;;  1. Arguments pathname and output-pathname 
;;;     - can be logical pathnames,
;;;     - are evaluated for for the whole book (top-level options of define-book)
;;;     - are not evaluated for book components.

(in-package :help)

(define-book YHELP.BOOK.EN
  :id "yhelp"                                   ; book_id
  :title "YstokHelp Concepts and User Guide"    ; book title
  :version "1.7.002"                            ; book/documentation version
  :language :en
  :pathname (pathname-location (current-pathname "en/"))
  :output-pathname (pathname-location (current-pathname "../deliver/help/en/"))
  :start "introduction"                         ; explicit start-topic name
  :environment                                  ; YHTML-Template variables and values
  ((product        . :help)
   (product-name   . "YstokHelp")               ; for stand-alone compiler application
   (version-string . "1.7.018")                 ; application version
   (asdlite-uri       . "http://lisp.ystok.ru/asdlite/")
   (asdf-uri          . "http://common-lisp.net/project/asdf/")
   (htt-uri           . "http://lisp.ystok.ru/yhtml/html-template.html")
   (yhelp-doc-uri     . "http://en.ystok.ru/products/yhelp/help/help.html")
   (yhelp-product-uri . "http://en.ystok.ru/products/yhelp/")
   (yhelp-src-uri     . "http://lisp.ystok.ru/yhelp/")
   (yuri-RM-uri       . "http://lisp.ystok.ru/yuri/help/help.html")
  )
  :dump nil                                    ; don't dump all book anchors by default
  :components
  (;(:aux-file "help.html")                    ; master frameset (explicit)
   (:frameset "help.html")                     ; master frameset (auto-generated)
   (:aux-file "help.js")
   (:aux-file "popupref.js")
   (:aux-file "popup.js")
   (:aux-file "navi.css")
   (:aux-file "cont.css")
   (:aux-folder "yhgfx")                        ; YstokHelp system folder with graphics
   (:navigation "index"                         ; file name of the overview in navi frame
    :tab "Contents"
    :description "Contents at a glance - chapter list")
   (:topic "yhusing")
   (:topic "introduction")
   (:topic "folders")
   (:popup "popup-note")                        ; demo pop-up note
 #|| Sorry, other topics/chapter has not translated to English yet...
   (:topic "aux-files")
   (:topic "navi-files")
   (:topic "cont-files")
   (:topic "popups")
   (:popup "bbcode-ppn"
    :pathname #P"PROJECTS:ynet;html;doc;en;bbcode-ppn")
   (:popup "popup-notes")                       ; all pop-up notes as segments
  ||#
   (:chapter compiler                           ; help book compiler
    :components 
    ((:topic "index")))
  #||(:topic "book-example")
     (:topic "help-scheme")
     (:topic "help-attribute")
     (:topic "segments")
     (:topic "specials")
     (:topic "define-book"
      :braces nil)                              ; prohibit braces in templ. pseudo-tags
     (:topic "component-classes")
     (:topic "popups")
     (:topic "gen-functions")
     (:topic "list-generator")
     (:topic "toc-compiler")
     (:topic "index-compiler"
      :braces nil)
     (:topic "book-compiler")                   ; :braces nil - in one segment only
     (:topic "localization")
   ))
   (:chapter run-time                           ; run-time API
    :dump t                                     ; save anchors of this chapter
    :components 
    ((:topic "index")
     (:topic "api")
     (:topic "context")))

   (:chapter main-board                         ; YstokHelp application
    :dump t                                     ; save anchors of this chapter
    :components 
    ((:topic "index")
     (:chapter settings                         ; application Settings dialog
      :dump t
      :components 
      ((:topic "index"                          ; General rules and screenshot
        :pathname #P"LIB:doc;en;settings;index.html")
       (:topic "general-tab")
       (:topic "anchors-tab")
       (:topic "components-tab")
       (:topic "logical-hosts")
       (:topic "html-tab"
        :pathname #P"PROJECTS:ynet;html;doc;en;html-tab")
       (:topic "help-tab")
       (:popup "lib-ppn"
        :pathname #P"LIB:doc;en;lib-ppn.html")
       ;(:context-popup "settings")             ; no meaningful text above tooltips
       (:context-popup "help-tab.cxt"           ; extension for avoiding component
        :pathname "help-tab.txt")))             ; name conflict      
     (:topic "command-line") ))

  ||# 
   (:aux-folder "gfx")                          ; YstokHelp application widgets
   (:topic "test1"                              ; this is only included when
    :features :test)                            ; :test is in *features*
   (:topic "copying")                           ; license on application/source code
   (:text-topic "license.txt"                   ; text version of license
    :pathname "license-template.txt"
    :title "YstokHelp text license"
    :description "The license generated from a plain text with YHTML-Template markup.")

   ;; Generated components not based on any source file
   (:index "idx"                                ; file name (prefix)
    :type :permuted
    :stopwords-language :en)
   (:toc "toc"                                  ; file name
    :max-level 3
    :source (:h1 :h2 (:h3))                     ; :title = collect from HTML-files title
    :heading-class (or :none "signature"))      ; headings CSS class pattern
  )
  :aliases                                      ; virtual anchors defined via origins
  (("book-descriptor" "BOOK-SYSTEM" "BOOK")))   ; (virtual origin1 origin2)
