;;; -*- Mode: Lisp; Encoding: Windows-1251 -*-
;;; YstokHelp - HTML-based help - Help book definition
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Copyright (c) 2009-2014 Ystok-Systema.  All rights reserved.
;;; Website: http://lisp.ystok.ru/yhelp/
;;;
;;; Sample multi-language definition of the help book about YstokHelp itself.
;;; Additionally includes example components for testing.
;;;
;;; NOTES 
;;;  1. Arguments pathname and output-pathname 
;;;     - can be logical pathnames,
;;;     - are evaluated for for the whole book (top-level options of define-book)
;;;     - are not evaluated for book components.

(in-package :help)

(define-book YHELP.BOOK.RU
  :id "yhelp"                                   ; book_id
  :version "1.7.015"                            ; book (documentation) version
  :language #-Russian :en
            #+Russian :ru
  :charset  #-Russian :iso-8859-1               ; Latin-1
            #+Russian "windows-1251"
  :pathname (pathname-location (current-pathname #-Russian "en/"
                                                 #+Russian "ru/"))
  :output-pathname (pathname-location (current-pathname
            #-(or test Russian)        "../deliver/help/en/"
            #+(and (not test) Russian) "../deliver/help/ru/"
            #+test                     "../deliver/help/test/"))
  :start "introduction"                         ; explicit start-topic name
  :environment                                  ; YHTML-Template variables and values
  ((product        . :help)
   (product-name   . "YstokHelp")               ; for stand-alone compiler application
   (version-string . "1.7.018")                 ; version of the application
   (asdlite-uri    . #-Russian "http://lisp.ystok.ru/asdlite/"
                     #+Russian "http://lisp.ystok.ru/ru/projects.html#ASDlite")
   (asdf-uri       . "http://common-lisp.net/project/asdf/")
   (htt-uri        . #-Russian "http://lisp.ystok.ru/yhtml/html-template.html"
                     #+Russian "http://lisp.ystok.ru/ru/yhtml/html-template.html")
   (yhelp-doc-uri     . #-Russian "http://en.ystok.ru/products/yhelp/help/help.html"
                        #+Russian "http://lisp.ystok.ru/ru/yhelp/help/help.html")
   (yhelp-product-uri . #-Russian "http://en.ystok.ru/products/yhelp/"
                        #+Russian "http://support.ystok.ru/downloads/")
   (yhelp-src-uri     . #-Russian "http://lisp.ystok.ru/yhelp/"
                        #+Russian "http://lisp.ystok.ru/ru/yhelp/")
   (yuri-RM-uri       . #-Russian "http://lisp.ystok.ru/yuri/help/help.html"
                        #+Russian "http://lisp.ystok.ru/yuri/help/help.html")
   (phtml . t)					; include chunks about PHTML
   #+test (test    . t)
  )
  #+test #+test
  :entities                                     ; pseudo-entities
  (("pe-string" . "<img class=\"glyph\" src=\"../gfx/pe-string.png\">")
   ("pe-lhtml"  . ((:img :class "glyph" :src "help:/gfx/pe-lhtml.png"))))

  :dump nil                                     ; do not dump all book anchors by default
  :components
  ((:aux-file "help.html")                      ; master frameset ( )
   ;(:frameset "help.html"                      ; master frameset (  .)
   ; #+Russian #+Russian :description "    ")
   (:aux-file "help.js")
   (:aux-file "popupref.js")
   (:aux-file "popup.js")
   (:aux-file "navi.css")
   (:aux-file "cont.css")
   (:aux-folder "yhgfx")                        ; YstokHelp system folder with graphics
   (:navigation "index"                         ; file name of the overview in navi frame
    :tab #-Russian "Contents"
         #+Russian ""
    :description #-Russian "Contents at a glance - chapter list"
                 #+Russian "  ,    ")
   (:topic "yhusing")
   (:topic "introduction")
   (:topic "folders")
   (:topic "aux-files")
   (:topic "navi-files")
   (:topic "cont-files")
   (:topic "popups")
   (:popup "popup-note")                        ; demo pop-up note
   (:popup "uri-ppn"
    :pathname #-Russian #P"PROJECTS:ynet;uri;doc;en;uri-ppn"
              #+Russian #P"PROJECTS:ynet;uri;doc;ru;uri-ppn")
   (:popup "htt-ppn"
    :pathname #-Russian #P"PROJECTS:ynet;html-template;doc;en;htt-ppn"
              #+Russian #P"PROJECTS:ynet;html-template;doc;ru;htt-ppn")
   (:popup "html-ppn"
    :pathname #-Russian #P"PROJECTS:ynet;html;doc;en;html-ppn"
              #+Russian #P"PROJECTS:ynet;html;doc;ru;html-ppn")
   (:popup "bbcode-ppn"
    :pathname #-Russian #P"PROJECTS:ynet;html;doc;en;bbcode-ppn"
              #+Russian #P"PROJECTS:ynet;html;doc;ru;bbcode-ppn")
   (:popup "popup-notes")                       ; all pop-up notes as segments
   (:chapter compiler                           ; help book compiler
    :components 
    ((:topic "index")
     (:topic "book-example")
     (:topic "help-scheme")
     (:topic "help-attribute")
     (:topic "segments")
     (:topic "specials")
     (:topic "define-book"
      :braces nil)
     (:topic "component-classes")
     (:topic "popups")
     (:topic "gen-functions")
     (:topic "list-generator")
     (:topic "toc-compiler")
     (:topic "index-compiler"
      :braces nil)
     (:topic "book-compiler")                   ; (:braces nil) - in one segment only
     (:topic "localization")
   ))
   (:chapter run-time                           ; run-time API
    :dump t                                     ; save anchors of this chapter
    :components 
    ((:topic "index")
     (:topic "api")
     (:topic "context")))

   (:chapter main-board                         ; YstokHelp application
    :dump t                                     ; save anchors of this chapter
    :components 
    ((:topic "index")
     (:chapter settings                         ; application Settings dialog
      :dump t
      :components 
      ((:topic "index"                          ; General rules and screenshot
        :pathname #-Russian #P"LIB:doc;en;settings;index.html"
                  #+Russian #P"LIB:doc;ru;settings;index.html")
       (:topic "general-tab")
       (:topic "anchors-tab")
       (:topic "components-tab")
       (:topic "logical-hosts")
       (:topic "html-tab"
        :pathname #-Russian #P"PROJECTS:ynet;html;doc;en;html-tab"
                  #+Russian #P"PROJECTS:ynet;html;doc;ru;html-tab")
       (:topic "help-tab")
       (:popup "lib-ppn"
        :pathname #-Russian #P"LIB:doc;en;lib-ppn.html"
                  #+Russian #P"LIB:doc;ru;lib-ppn.html")
       ;(:context-popup "settings")             ; no meaningful text above tooltips
       (:context-popup "help-tab.cxt"           ; extension for avoiding component
        :pathname "help-tab.txt")))             ; name conflict      
     (:topic "command-line") ))

   (:aux-folder "gfx")                          ; YstokHelp application widgets

   ;; Example components - please ignore
   (:topic "test1" :features :test)
   (:topic "test2" :features :test)
   (:topic "test-popups" :features :test        ;:popups (:note :menu)
           :braces nil)
   (:popup "test-popup1" :features :test)
   (:popup "test-popup2" :features :test)
   (:chapter chapter-one :features :test
    :pathname "chapter1/"                       ; trailing slash '/' is optional
    ;:start "test11"
    :components
    (;(:toc "index")                            ; chapter toc in navi frame is overkill?
    (:topic "test11")
    (:chapter chapter-one-1                     ; virtual chapter
     :pathname #P"PROJECTS:yhelp;doc;"          ; gets sources from outside the book
     :output-pathname "."                       ; puts results just into the parent folder
     :components
     ((:topic "outside-test-page")))))
   (:chapter chapter2 :features :test
    :depends-on ("introduction" chapter-one)
    :components
    ((:topic "test22")))

   (:topic "copying")                           ; application/source code license

   ;; Generated components not based on any source file
   (:index "idx"                                ; file name (prefix)
    #+Russian #+Russian :tab ""
    #+Russian #+Russian :title " "
    #+Russian #+Russian :description "  -       "
    :type :permuted
    :stopwords-language (:en #+Russian :ru))
   (:toc "toc"                                  ; file name
    #+Russian #+Russian :tab ""
    #+Russian #+Russian :title ""
    #+Russian #+Russian :description "   "
    :max-level 3
    :source (:h1 :h2 (:h3))                     ; :title = collect from HTML-files title
    :heading-class (or :none "signature"))      ; headings CSS class pattern
  )
  :aliases                                      ; virtual anchors defined through origins
  (("book-descriptor" "BOOK-SYSTEM" "BOOK")))   ; (virtual origin1 origin2)
