;;; -*- Mode: Lisp; -*-
;;; YstokHelp - HTML-based help
;;; Copyright (c) 2009-2014 Dr. Dmitry Ivanov.  All rights reserved.
;;; Website: http://lisp.ystok.ru/yhelp/
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Computing column or grid anchor, menu construction

(in-package :ystok.help)

(defun %grid-locator (grid &optional context
                                     (column (grid:current-column grid))
                                     (parent (%object-locator grid context)))
 ;;; Value: List of strings (locator1 locator2 ...)
  ;; Q: Is there any sense to union with grid and interface locators for context?
  (let ((help-key (and column (grid:help-key column))))
    (labels ((%atom-locator (arg &optional fragment)	; -> string or symbol (m.b. NIL)
               (cond ((stringp arg)
                      arg)
                     ((numberp arg) 
                      (princ-to-string arg))
 		     ;; Below code is similar to %pane-locator
                     ((and arg (symbolp arg))
                      (case arg				; only if explicit
                        ((t :name)			; T means column-name
                         (let ((value (grid:column-name column)))	  ; name symbol
                           (if fragment (coerce-to-anchor value) value))) ; must be kept
                        (:interface
                         (let ((interface (capi:top-level-interface grid)))
                           (if (and fragment (consp (capi:help-key interface)))
                               (type-of interface)	; path unacceptable-coerce to :type
                               (first-or-self (help-locator interface context)))))
                        ((:parent :frame)
                         parent)
                        (:tab				; ys-product only
                         (first-or-self
                          (help-locator (ywi::ambient-layout-visible-child grid) context)))
                        (otherwise
                         (if fragment (coerce-to-anchor arg) arg)))) ; symbol must be kept
                     ((vectorp arg)
                      (interleave-seq 'string #\_	; symbols -> symbol names
                                      (remove nil (map 'list #'(lambda (arg)
                                                                 (%atom-locator arg t))
                                                             arg)))))))
      (%union-locator (cond ((consp help-key)
                             (%locator (help-key-id help-key context)
                                       #'%atom-locator context))
                            ((not (stringp help-key))	; not pointer-documentation-enter
                             (when-let (atom (%atom-locator help-key))
                               (string atom))))
                      parent
                      (help-locator (capi:top-level-interface grid) context)))))
                      ;(mapcar (lambda (arg) (%locator arg #'%atom-locator))
                      ;        (interface-default-locator
                      ;         (capi:top-level-interface grid)))))))

(defmethod help-locator ((pinboard grid::record-board) &optional context)
  (%grid-locator (capi:element-interface pinboard) context
                 (ywi:current-displayer pinboard)))

(defmethod help-locator ((grid grid:ygrid) &optional context)
  (%grid-locator grid context))

(defmethod grid:cell-popup-menu-items :around ((grid grid:ygrid) (record grid:record)
                                               (column grid:column) &optional edit)
  (declare (ignore edit))
  (when-let (items (call-next-method))
    (if (yl:thys-preference '*show-what-is-this-mi*)
        (let ((help-key (or (help-key column) (capi:help-key grid))))
          (nconc items
          `(,(make-instance 'capi:menu-component :items
             `(,(make-instance 'capi:menu-item
		:mnemonic-title #L"&What's This?"
	        :name #1=:help :data #1#		; :contexthelp would be better
                :enabled-function (if help-key #'lw:true #'lw:false)
	        :callback-type :interface
	        :callback (lambda (interface)
                            (popup-context (grid:pinboard grid)
                                           :displayer column
                                           :interface interface
                                           :help-key help-key))
                            ;(ywi:help-callback interface pane #1# help-key))
		:help-key nil))))))
        items)))

