;;; -*- Mode: Lisp; -*-
;;; YstokHelp - HTML-based help
;;; Copyright (c) 2009-2014 Dr. Dmitry Ivanov.  All rights reserved.
;;; Website: http://lisp.ystok.ru/yhelp/
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Perform and other complex methods

(in-package :ystok.help)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  GENERATED-COMPONENT  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod asd:perform :before ((o (eql :compile)) (c generated-component))
  (multiple-value-bind (pathname created)
      (ensure-directories-exist (asd:component-output-pathname c))
    (when created
      (log-info "Created directory ~a" (pathname-location pathname)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  FRAMESET  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod asd:perform ((o (eql :compile)) (c frameset))
  (let* ((pathname (asd:component-output-pathname c))
         (book (asd:component-system c))
         (lang (language book))
         (charset (charset book))
         (navigation (find 'navigation-mixin (asd:module-components book) :test #'xtypep))
         (start-topic (start-topic book)))
   (html:with-html-file (pathname
                         :external-format (output-html-format book)
                         :mode :xml
                         :prologue (if charset
                                       (string-append
                                        (format nil html::+xml-declaration-format+ charset)
                                        #\Newline
                                        html::+xhtml-frameset-dtd-string+)
                                       html::+xhtml-frameset-dtd-string+))
    (log-info "creating frameset-> ~a" pathname)
    (html:html (:lang (:optional lang)
                :xml\:lang (:optional lang)
                :xmlns html::+xmlns+)
     (terpri html:*html-stream*)
     (html:head ()
       (html:title ()
         (html:princ-safe-http (or (title c) (title book) (asd:component-name book))))
       (html:meta (:http-equiv :content-type
                   :content (format nil "text/html~@[;charset=~a~]" charset)))
       ;(when lang
       ;  (html:meta (:http-equiv :content-language
       ;              :content (string-downcase lang))))|#
       (html:meta (:name :description
                   :content (or (description c)
                                (description book)
                                "Help book frameset definition")))
       (html:meta (:name "ROBOTS" :content "NOINDEX"))
       (html:meta (:name "GENERATOR" :content #+ys-product (string-append "YstokHelp "
                                                             ys:*thys-version-string*)
                                              #-ys-product "YstokHelp"))
       (html:script (:src "help.js" :type "text/javascript")))
     (terpri html:*html-stream*)
     (html:frameset (:id "yh_frameset" :cols "33%,*"
                     :onload "frameset_onload()" :onunload "frameset_onunload()")
       (html:frame (:src (if navigation
                             #1=(uri:enough-uri (asd:component-output-pathname navigation)
                                                pathname)
                             "index.html")
                    :name "navi" :id "navi"
                    :title (gettext :navigation *yhelp-bundle* lang)
                    :frameborder 1 :scrolling :yes))
       (html:frame (:src #2=(uri:enough-uri (asd:component-output-pathname start-topic) pathname)
                    :name "cont" :id "cont"
                    :title (gettext :content *yhelp-bundle* lang)
                    :scrolling :yes))
       (terpri html:*html-stream*)
       (html:noframes ()				; a la toolbar-lhtml
         (html:p ()
           (when navigation
             (html:a (:href #1#
                      :title (description navigation))
               (html:princ-safe-http (tab navigation)))			; "Contents"
             (html:princ-http " | "))
           (when-let (index (find-index book))
             (html:a (:href (uri:enough-uri (index-start-pathname index) pathname)
                      :title (description index))
               (html:princ-safe-http (tab index)))			; "Index"
             (html:princ-http " | "))
           (when-let (toc (find-toc book))
             (html:a (:href (uri:enough-uri (asd:component-output-pathname toc) pathname)
                      :title (description toc))
               (html:princ-safe-http (tab toc)))))			; "Sitemap"
         (html:p ()
           (html:a (:href #2# :title (:optional (description start-topic)))
             (html:princ-safe-http (title start-topic)))) ))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  NAVIGATION  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *navi-tabs-div-css-class* "toolbar header")

(defun navi-html-head (book &key level title
                                 (target "cont")
                                 (charset (charset book)))
                                 ;(language (language book)))
 ;;; Output the HEAD element of a generated navi component directly to *html-stream*
  ;; Args: level  Physical level in directory structure, can differ from component-level!
  ;;       target The name tagret frame
  (html:head ()
    (html:title () (html:princ-safe-http title))
    ;; A la ensure-charset-language-lhtml
    (html:meta (:http-equiv :content-type
                :content (format nil "text/html~@[; charset=~a~]" charset)))
    ;(when language
    ;  (html:meta (:http-equiv :content-language
    ;              :content (string-downcase language))))
    (html:meta (:name "ROBOTS" :content "NOINDEX"))
    (html:meta (:name "GENERATOR"
                :content #+ys-product (string-append "YstokHelp " ys:*thys-version-string*)
                         #-ys-product "YstokHelp"))
    (when target
      (html:base (:target target)))
    (html:link (:href "navi.css" :rel "stylesheet" :type "text/css"))
    (html:script (:src "help.js" :type "text/javascript"))
    (html:htm
     (:jscript (:format "navi_onopen(~d~@[,~s~]);" (or level 0) nil)))) ;default-topic
  (terpri html:*html-stream*))

(defmethod head-cont-lhtml ((c navigation) old-cont &key)
 ;;; Return LHTML list for the HEAD element of the file-based navigation component
  ;; Args: level Not implemented - only top-level for now.
  (let ((help.js (ensure-aux-link-lhtml :script "help.js" old-cont c)))
    `(,@(ensure-charset-lhtml old-cont c)
      ,@(unless (html:find-element :base old-cont)
         `(((:base :target "cont"))))
      ,@(ensure-aux-link-lhtml :link "navi.css" old-cont c)
      ,@help.js
      ,@(when help.js					; no help.js - no onopen call
          `((:jscript "navi_onopen();")
            :newline)) )))

(defun navi-tabs-lhtml (self &optional (book (asd:component-system self))
                                       (components (remove 'navigation-mixin
                                                           (asd:module-components book)
                                                           :test-not #'xtypep)))
 ;;; Create tabs-like toolbar for pages in the navi frame.
  ;; Args: self       The component the toolbar is about to be created for.
  ;;       components Components to put onto the tabs.
  ;;		      By default top-level instances of the navigation-mixin are gathered. 
  ;;		      They appear is the same order as in the book definition.
  ;; Value: LHTML list
  ;; NB: The tabs order is defined by the order of the corresponding components
  ;;	 in the book system definition.
  `(((:div :class ,*navi-tabs-div-css-class*)
     ,.(loop for c in components
             and first = t then nil
             unless first collect " | " end
             collect (if (eq c self)
                         `((:span :class :current)
                           ,(tab c))
                         `((:a :href ,(uri:enough-uri (asd:component-output-pathname c)
                                                      (asd:component-output-pathname book))
                            :target "_self"
                            :title ,(description c))
                           ,(tab c)))))))

(defmethod toolbar-lhtml ((component navigation) (position (eql :header)) segment
                          &key content (book (asd:component-system component)))
 ;;; Adjoin only if there is not an explicit toobar in the source HTML file.
  (declare (ignore segment position))
  (unless (html:find-element :div content :class *navi-tabs-div-css-class*
                             :test #'string-equal)
    (navi-tabs-lhtml component book)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  TOPIC  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod rel-object ((topic topic) (rel (eql :prev)))
  (let* ((topics (topics (asd:component-system topic)))
         (position (position topic topics)))
    (when (iplusp position)
      (last-segment (nth (1- position) topics)))))

(defmethod rel-object ((segment segment) (rel (eql :prev)))
  (let ((topic (source-component segment))
        (number (segment-number segment)))
    (if (= number 1)
        topic
        (nth-segment (1- number) topic))))

(defmethod rel-object ((topic topic) (rel (eql :next)))
  (or (first (segments topic))				; first genuine segment of itself
      (first-segment (second (memq topic (topics (asd:component-system topic)))))))

(defmethod rel-object ((segment segment) (rel (eql :next)))
  (let ((topic (source-component segment))
        (number (segment-number segment)))
    (if (< number (length (segments topic)))
        (nth-segment (1+ number) topic)
        (first-segment (second (memq topic (topics (asd:component-system topic))))))))
                       ;(first (member-if (lambda (c))
                       ;       (rest (memq topic (topics (asd:component-system topic)))))
        ;(first-segment (rel-object topic rel)))))

(defmethod rel-object ((topic topic) (rel (eql :up)))
  (let ((parent (asd:component-parent topic)))
    (when-let (start-topic (start-topic parent))
      (when (cond ((memq start-topic (memq topic (topics parent)))
                   ;; this topic is start-topic itself or preceeds it, e.g. using.html
                   ;(eq start-topic topic)				; bump into itself
                   (and (setq parent (asd:component-parent parent))	; - jump to parent
                        (setq start-topic (start-topic parent))))
                  (t))
        (first-segment start-topic)))))

(defmethod rel-object ((segment segment) (rel (eql :up)))
  (let ((topic (source-component segment)))
    (if (eql (segment-number segment) 0)
        (rel-object topic rel)
        ;; Decide whehter the first segment is "upper" for this one by comparing toc-levels
        ;; If the heading level of the segment too low, the heading was not collected
        (let* ((toc (find-toc (asd:component-system topic)))
               (toc-source (if toc (toc-source toc) :title))
               (first-segment (first-segment topic))
               (first (if (eq toc-source :title)		; level of zero segment
                          (segment-title-toc-level first-segment)
                          (heading-level (first (headings topic)))))
               (self-level (if (eq toc-source :title)		; level of segment given
                               (segment-title-toc-level segment)
                               (when-let (heading (first (memq segment (headings topic)
                                                               :key #'heading-segment)))
                                 (heading-level heading)))))
          (if (and first (imp self-level (< first self-level)))	; if of lower level,
              first-segment					; up redirects to zero
              (rel-object topic rel)))))) ; all segments are assumed of the same level

(defmethod head-cont-lhtml ((c topic) old-cont
                            &key level (popups (asd:component-property c :popups)))
 ;;; Q: Replace Content-Type with book output charset and language?
  (let ((help.js (ensure-aux-link-lhtml :script "help.js" old-cont c)))
    `(,@(ensure-charset-lhtml old-cont c)
      ,@(ensure-aux-link-lhtml :link "cont.css" old-cont c)
      ,@help.js
      ,@(when popups
          (ensure-aux-link-lhtml :script "popupref.js" old-cont c))
      ,.(when help.js					; no help.js - no onopen call
          `((:jscript ,(format nil "cont_onopen(~@[~a~]);"
                            (or level (chapter-folder-level (asd:component-parent c)))))
            :newline)) )))

(defmethod toolbar-lhtml ((topic topic) position segment
                          &key (book (asd:component-system topic)))
 ;;; Args: segment  Segment instance or topic itself
  ;;       content  Body content
  (when-let (description (case position
                          (:header (slot-value book 'content-header))
                          (:footer (slot-value book 'content-footer))))
    `(((:div :class ,(format nil "~(~a~) toolbar" position))
       (:ul
        ,@(do ((divider t)
               (language (language book))
               (acc ())
               rel
               (rest description (rest rest)))
              ((null rest)
               (nreverse acc))
            (case (setq rel (first rest))
              (:toggle ;:sync
               (push `(,(if divider '(:li :class "divider") :li)
                       ((:a :id ,(format nil "yh_navi~(~a~)" rel)
                         :href "#"
                         :onclick ,(format nil "navi~(~a~)_onclick(event)" rel)
                         :title ,(gettext `(:navi ,rel :title) *yhelp-bundle* language))
                        :nbsp))
                     acc))
              (otherwise
                (let ((text (gettext rel *yhelp-bundle* language)))
                            ;(cdr-assoc rel *rel-text-alist*)
                  (push `(:li
                          ,(if-bind (rel-object (rel-object segment rel))
                             `((:a :class ,rel :rel ,rel
                                :title ,(title rel-object)
                                :href ,(uri:enough-uri (asd:component-output-pathname rel-object)
                                                       (asd:component-output-pathname segment)))
                               ,text)
                             `((:span :class ,rel) ,text)))
                        acc))) ))))
      :newline)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  POPUP  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod h-callback-alist ((popup popup) &optional toc-source)
                                           ;(toc-max-level (toc-max-level book)))
 ;;; Args: *current-heading*  NIL means asd:load-source-op
  ;;			      non-NIL - asd:compile-op
  ;; Used as: parse-html :callback argument
  (declare (ignore toc-source))
  (when (and *index-heading-anchors* (not *current-heading*))	; load-source-op
    (loop for key across +heading-keys+
          collect `(,key . collect-anchors-callback))))

(defmethod head-cont-lhtml ((c popup) old-cont &key)
  (let ((popup.js (ensure-aux-link-lhtml :script "popup.js" old-cont c)))
    `(,@(ensure-charset-lhtml old-cont c)
      ,@(unless (html:find-element :base old-cont)
         `(((:base :target "_parent"))))
      ,@(ensure-no-inverse-elem :meta old-cont :name "ROBOTS" :content "NOINDEX")
      ,@(ensure-aux-link-lhtml :link "cont.css" old-cont c)
      ,@popup.js
      ,@(when popup.js
          `((:jscript "popup_onopen();")
            :newline)) )))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  CHAPTER  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun chapter-folder-level (chapter)
 ;;; Depth of the chapter within the physical directory structure of the book
  (- (length (pathname-directory (asd:component-output-pathname chapter)))
     (length (pathname-directory (asd:component-output-pathname (asd:component-system chapter))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  BOOK  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmacro define-book (name &body args &key pathname output-pathname
                                            (default-component-class 'topic)
                            &allow-other-keys)
 "Define and register help book NAME. Arguments:
ID - book id, string or symbol, defaults to system name coerced to string.
Other arguments are specified just like for ASD:DEFSYSTEM"
  ;; NB: Book redefinition returns the same book instance where
  ;;	 - some of its components are the same,
  ;;	 - some of them are replaced by new instances.
  ;; ASDF NB:
  ;;	 We overcome ASDF drawback - :default-component-class must can only be
  ;;	 specified in defsystem explicity, not via :default-initargs in defclass.
  (unless output-pathname
    (asd::sysdef-error #L"Option ~s is required for define-book." :output-pathname))
  `(apply #'asd:ensure-system ',name 'book-system
          :pathname ,(or pathname			; evaluated
                         `(if *load-truename*
                              (pathname-location *load-truename*)
                              *default-pathname-defaults*))
          :output-pathname ,output-pathname		; evaluated
          :default-component-class ',default-component-class
          ',(remove-properties args			; other options are not evaluated
                               '(:class :pathname :output-pathname
                                 :default-component-class))))

(defmethod find-book ((name symbol) &optional (errorp t))
  (asd:find-system name errorp))

(defmethod asd:perform :after ((o (eql :compile)) (book book-system))
  (log-info "Done help book-> ~a - ~s~%" (asd:component-name book) (id book)))

(defun compile-book (name &rest args &key force &allow-other-keys)
 ;;; Do we need this?
  ;; Args: :index t :toc t :load t :force t
  (let ((*current-book* (if (typep name 'book-system) name (asd:find-system name))))
    (when force
      (clrhash (anchors *current-book*))
      (clrhash (text-anchors  *current-book*))
      (%initialize-anchors *current-book*))		; put aliases to hash-table
    (apply #'asd:operate :compile *current-book* args)))

(defmethod dump-anchors ((book-system book-system)
                         &optional (pathname (make-pathname
                                              :name (id book-system)
                                              :type *anchor-file-type*
                                              :defaults (asd:component-output-pathname
                                                         book-system))))
 ;;; Save anchors into the file excluding those defined in popups.
  ;; The file contains lists of the form
  ;;	(anchor . uri-path) or
  ;;	(anchor uri-desc1 uri-desc2 ...)
  ;; where uri-desc ::= uri-path
  ;;		      | (uri-path . segment-title)
  ;;		      | (namestring start . end)
  ;; and each uri-path or namestring is a physical path relative to the book folder.
  ;; NB: There is a little rationale to implement logical path juxtaposition
  ;;     unless we also implement a kind of "**" wild-card matching any
  ;;     part of path though the chapter/folder structure.
  (let ((base (asd:component-output-pathname book-system))
        (external-format (external-format book-system)))
    (log-info "Dumping anchors: ~a" pathname)
    (with-open-file (stream pathname :direction :output
                            :if-exists :supersede
                            :external-format external-format)
      ;; Print header: comment and plist
      (write-string ";;; Help Book Anchors" stream)
      (print `(:id ,(id book-system)
               ,@(when-let (start-topic (start-topic book-system))
                   `(:start ,(uri:render-uri (uri:enough-uri
                                              (asd:component-output-pathname (first-segment
                                                                    start-topic))
                                              base)
                                             nil external-format)))
               ,@(when-let (title (title book-system))
                   `(:title ,title)))
             stream)
      (loop with anchors = (anchors book-system)
            for anchor being the hash-key of anchors using (hash-value saplist)
            for aplist =					; desend to original
                ;(remove-if #'popup-p				; real topics only
                        (remove-if-not (lambda (aplist)
                                         (getf (rest aplist) :dump (dump (first aplist))))
                                       (nconc (mappend (lambda (origin)
                                                         (anchor-saplist origin anchors))
                                                       (remove-if-not #'stringp saplist))
                                              (remove-if-not #'consp saplist)))
                ;           :key #'car)
            for tuple =
               (cond ((rest aplist)
                      (loop for (segment)
                            in (sort (copy-list aplist) #'segment< :key #'car)
                            for string = (uri:render-uri
                                          (uri:enough-uri
                                           (asd:component-output-pathname segment) base)
                                          nil external-format)
                            collect (if-bind (title (segment-property segment :title
                                                                      nil t)) ;:source
                                      (cons string title)
                                      string)))
                     (aplist
                      (uri:render-uri (uri:enough-uri
                                       (asd:component-output-pathname (caar aplist)) base)
                                      nil external-format)))
            when tuple 
            do (terpri stream)
               (prin1 (cons anchor tuple) stream))
      (print :text stream)					; context separator
      (loop with text-anchors = (text-anchors book-system)
            for anchor being the hash-key of text-anchors using (hash-value alist)
            for tuple = (loop for pair in alist			; replace component by path
                              collect (cons (uri:render-uri (uri:enough-uri
                                                             (asd:component-output-pathname
                                                              (car pair))
                                                             base)
                                                            nil external-format)
                                            (cdr pair)))
            do (terpri stream)
               (prin1 (cons anchor tuple) stream)))
    pathname))

(defmethod dump-anchors (book-system &optional (pathname nil supplied-p))
  (apply #'dump-anchors (asd:find-system book-system t)
         (when supplied-p (list pathname))))

