;;; -*- Mode: Lisp; -*-
;;; YstokHelp - HTML-based help
;;; Copyright (c) 2009-2014 Dr. Dmitry Ivanov.  All rights reserved.
;;; Website: http://lisp.ystok.ru/yhelp/
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Text topics compiling and expanding templates

(in-package :ystok.help)

;;; Text-topic does not assume any parsing for now
;;; Q: Convert to book external-format?

(defclass text-topic (topic-mixin parsed-file) ())
 ;((title :initarg :title :reader title :initform nil)))		; for TOC

(defmethod segments ((self text-topic)) ())

(defmethod reinitialize-instance :after ((self text-topic)
                                         &key (toc-level *unbound-value*)
                                              (use-templates t supplied-p))
  (if (eq toc-level *unbound-value*)
      (remf (asd:component-properties self) :toc-level)
      (setf (asd:component-property self :toc-level) toc-level))
  (if supplied-p
      (setf (asd:component-property self :use-templates) use-templates)
      (remf (asd:component-properties self) :use-templates)))

(defmethod asd:source-file-type ((c text-topic) s)
  (or (asd::source-file-explicit-type c) "txt"))

(defmethod asd:perform ((o (eql :preprocess)) (c text-topic))
  (declare (ignore o))
  (setf (headings c) ()))

(defmethod asd:output-files ((o (eql :compile)) (c text-topic))
  (declare (ignore o))
  (list (asd:component-output-pathname c)))

(defmethod asd:perform ((o (eql :compile)) (c text-topic))
 ;;; For now, treat as aux-file but put the title into the heading list
  (let* ((toc-level (asd:component-property c :toc-level *unbound-value*))
         (input-file (asd:component-pathname c))
         (output-file (asd:component-output-pathname c))
         (book-system (asd:component-system c))
         (use-templates (if (use-templates book-system)
                            (asd:component-property c :use-templates
                                                    (use-templates book-system))
                            nil)))
    (setf (headings c)
          (when toc-level
            (list (make-heading :segment c			; no tag
                                :level (if (eq toc-level *unbound-value*)
                                           (segment-toc-level c nil)
                                           toc-level)
                                :id nil
                                :content (list (title c))))))
    (if use-templates
        (let* ((htt:*default-template-pathname* (pathname-location output-file))
               (htt:*template-package* (load-time-value (find-package :ystok.help)))
               (htt:*template-symbol-package* htt:*template-package*)
               (htt:*accept-braces-markers* (eq use-templates :braces)))
          (log-info "compile: ~a" input-file)
          (with-open-file (stream output-file :direction :output
                                  :external-format (output-html-format book-system)
                                  :if-exists :supersede)
            (htt:fill-and-print-template input-file (environment book-system) 
                                         :stream stream
                                         :external-format (external-format book-system))
            (log-info "compile-> ~a" output-file)))
        (log-copy-file input-file output-file))))

